package hdm.mi.growbros.repositories;

import hdm.mi.growbros.models.GardenEntry;
import hdm.mi.growbros.models.plant.Plant;
import org.junit.jupiter.api.BeforeAll;
import org.junit.jupiter.api.Test;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.context.SpringBootTest;

import java.util.List;

import static org.junit.jupiter.api.Assertions.assertEquals;

//TODO: Test for when the current week is in between plant/harvest start and end date

@SpringBootTest
class GardenRepositoryTest {
    @Autowired
    private GardenRepository underTest;

    private static GardenEntry ge1, ge2, ge3;

    @BeforeAll
    public static void setupAll(@Autowired GardenRepository underTest, @Autowired PlantRepository plantRepository) {
        Plant p1 = Plant.builder()
                .name("Early plant")
                .plantWeekStart(1)
                .plantWeekEnd(3)
                .harvestWeekStart(10)
                .harvestWeekEnd(15)
                .build();
        Plant p2 = Plant.builder()
                .name("Medium late plant")
                .plantWeekStart(4)
                .plantWeekEnd(6)
                .harvestWeekStart(18)
                .harvestWeekEnd(20)
                .build();
        Plant p3 = Plant.builder()
                .name("Late plant")
                .plantWeekStart(10)
                .plantWeekEnd(15)
                .harvestWeekStart(25)
                .harvestWeekEnd(30)
                .build();
        ge1 = new GardenEntry(0L, p1, null);
        ge2 = new GardenEntry(0L, p2, null);
        ge3 = new GardenEntry(0L, p3, null);
        plantRepository.saveAll(List.of(p1, p2, p3));
        underTest.saveAll(List.of(ge1, ge2, ge3));
    }

    @Test
    void testNearestHarvestWeek() {
        //when
        var result = underTest.findAllByNearestHarvestWeek(1);

        //then
        assertEquals(List.of(ge1, ge2, ge3), result);
    }

    @Test
    void findNearestPlantingWeek() {
        //when
        var result = underTest.findAllByNearestPlantingWeek(1);

        //then
        assertEquals(List.of(ge1, ge2, ge3), result);
    }
}